# flake8: noqa: F401

from .generic import (
    CreateView,
    DeleteView,
    DeleteObjectsView,
    DetailView,
    FormView,
    HistoryView,
    ListView,
    ModelFormView,
    ModelView,
    ObjectFormView,
    ObjectView,
    ObjectsFormView,
    ObjectsView,
    TemplateView,
    UpdateView,
    View,
)
